
import Foundation
import HyperSnapSDK

public class FLHVDocsCapture{
    
    var docConfig = HyperSnapSDK.HVDocConfig()
    
    public func getDocConfig() -> HVDocConfig {
        return self.docConfig
    }
    
    public func createNewDocConfig() {
        docConfig = HyperSnapSDK.HVDocConfig()
    }
    
    public func setHVDocConfig(hvDocConfig: Dictionary<String, Any>) {
        if let aspectRatioValue = (hvDocConfig[FLHyperSnapConstants.DOC_ASPECT_RATIO_VALUE] as? Double){
            getDocConfig().setAspectRatio(aspectRatioValue)
        }
        if let shouldSetPadding = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SET_PADDING] as? Bool){
            getDocConfig().setShouldAddPadding(shouldSetPadding)
        }
        if let defaultPaddingValue = (hvDocConfig[FLHyperSnapConstants.DOC_DEFAULT_PADDING_VALUE] as? Double){
            getDocConfig().setDefaultPadding(defaultPaddingValue)
        }
        if let shouldShowFlashIcon = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_FLASH_ICON] as? Bool){
            getDocConfig().setShouldShowFlashButton(shouldShowFlashIcon)
        }
        if let shouldShowReviewScreen = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_REVIEW_SCREEN] as? Bool){
            getDocConfig().setShouldShowReviewPage(shouldShowReviewScreen)
        }
        if let shouldEnableDocUpload = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_ENABLE_UPLOAD] as? Bool){
            getDocConfig().setEnableDocumentUpload(shouldEnableDocUpload)
        }
        if let shouldShowInstructionPage = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_INSTRUCTION_PAGE] as? Bool){
            getDocConfig().setShouldShowInstructionsPage(shouldShowInstructionPage)
        }
        if let shouldHandleRetries = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_ENABLE_RETRIES] as? Bool){
            getDocConfig().setShouldHandleRetries(shouldHandleRetries)
        }
        if let shouldShowCloseAlert = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_CLOSE_ALERT] as? Bool){
            getDocConfig().setShouldShowCloseAlert(shouldShowCloseAlert)
            if (shouldShowCloseAlert){
                if let closeAlertDialogTitle = (hvDocConfig[FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_TITLE] as? String){
                    if(!closeAlertDialogTitle.isEmpty){
                        getDocConfig().textConfig.setCloseAlertTitle(closeAlertDialogTitle)
                    }
                }
                if let closeAlertDialogDescription = (hvDocConfig[FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_DESCRIPTION] as? String){
                    if(!closeAlertDialogDescription.isEmpty){
                        getDocConfig().textConfig.setCloseAlertDesc(closeAlertDialogDescription)
                    }
                }
            }
        }
        
        if let docType = (hvDocConfig[FLHyperSnapConstants.DOC_TYPE_VALUE] as? String){
            let documentType = getDocumentType(document: docType)
            getDocConfig().setDocumentType(documentType)
        }
        
        if let shouldShowTrustLogos = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_TRUST_LOGOS] as? Bool){
            getDocConfig().setShowTrustLogos(shouldShowTrustLogos)
        }
        
        if let ocrApiDetails = (hvDocConfig[FLHyperSnapConstants.OCR_API_DETAILS] as? Dictionary<String, Any>){
            if (!ocrApiDetails.isEmpty){
                setOCRAPIDetails(ocrApiDetails: ocrApiDetails)
            }
        }
        
        if let uiStrings = (hvDocConfig[FLHyperSnapConstants.DOC_CAPTURE_UI_STRINGS] as? Dictionary<String, Any>){
            if (!uiStrings.isEmpty){
                setUIStrings(uiStrings: uiStrings)
            }
        }
        if let shouldAutoCapture = (hvDocConfig[FLHyperSnapConstants.DOC_AUTO_CAPTURE] as? Bool){
            if (shouldAutoCapture) {
                if let duration = (hvDocConfig[FLHyperSnapConstants.DOC_AUTO_CAPTURE_DURATION] as? Int){
                    getDocConfig().setAutoCaptureDuration(durationInMS: duration)
                }
            }
            getDocConfig().setShouldAutoCapture(shouldAutoCapture)
        }
        
    }
    
    public func setOCRAPIDetails(ocrApiDetails: Dictionary<String, Any>) {
        
        let ocrEndpoint = ocrApiDetails[FLHyperSnapConstants.OCR_ENDPOINT] as! String
        let documentSideString = ocrApiDetails[FLHyperSnapConstants.DOCUMENT_SIDE_STRING] as! String
        let documentSide = getDocumentSide(documentSide: documentSideString)
        let params = ocrApiDetails[FLHyperSnapConstants.PARAMS] as? Dictionary<String, Any> ?? [String: Any]()
        let headers = ocrApiDetails[FLHyperSnapConstants.HEADERS] as? Dictionary<String, Any> ?? [String: Any]()
        
        getDocConfig().setOCRAPIDetails(ocrEndpoint, documentSide: documentSide, params: params as [String : AnyObject], headers: headers as? [String : String])
        
    }
    
    public func setUIStrings(uiStrings: Dictionary<String,Any>) {
        if let keyExists = uiStrings["docInstructionsTitle"] as? String {
            getDocConfig().textConfig.setDocInstructionsTitle(keyExists)
        }
        if let keyExists = uiStrings["docInstructions1"] as? String{
            getDocConfig().textConfig.setDocInstructions1(keyExists )
        }
        if let keyExists = uiStrings["docInstructions2"] as? String{
            getDocConfig().textConfig.setDocInstructions2(keyExists )
        }
        if let keyExists = uiStrings["docInstructions3"] as? String{
            getDocConfig().textConfig.setDocInstructions3(keyExists )
        }
        if let keyExists = uiStrings["docInstructionsProceed"] as? String{
            getDocConfig().textConfig.setDocInstructionsProceedText(keyExists )
        }
        if let keyExists = uiStrings["docCaptureTitle"] as? String{
            getDocConfig().textConfig.setDocCaptureTitle(keyExists )
        }
        if let keyExists = uiStrings["docCaptureSubtitle"] as? String{
            getDocConfig().textConfig.setDocCaptureSubtitle(keyExists )
        }
        if let keyExists = uiStrings["docCaptureDescription"] as? String{
            getDocConfig().textConfig.setDocCaptureDescription(keyExists )
        }
        if let keyExists = uiStrings["docCaptureSubText"] as? String{
            getDocConfig().textConfig.setDocCaptureSubText(keyExists )
        }
        if let keyExists = uiStrings["docReviewTitle"] as? String{
            getDocConfig().textConfig.setDocReviewTitle(keyExists )
        }
        if let keyExists = uiStrings["docReviewSubtitle"] as? String{
            getDocConfig().textConfig.setDocReviewSubtitle(keyExists )
        }
        if let keyExists = uiStrings["docReviewDescription"] as? String {
            getDocConfig().textConfig.setDocReviewDescription(keyExists )
        }
        if let keyExists = uiStrings["docReviewRetakeButton"] as? String{
            getDocConfig().textConfig.setDocReviewRetakeButtonText(keyExists )
        }
        if let keyExists = uiStrings["docReviewContinueButton"] as? String{
            getDocConfig().textConfig.setDocReviewContinueButtonText(keyExists )
        }
        if let keyExists = uiStrings["docLoaderTitle"] as? String{
            getDocConfig().textConfig.setDocLoadingTitleText(keyExists )
        }
        if let keyExists = uiStrings["docLoaderDesc"] as? String{
            getDocConfig().textConfig.setDocLoadingDescriptionText(keyExists )
        }
        if let keyExists = uiStrings["docRetakeTitleText"] as? String{
            getDocConfig().textConfig.setDocRetakePageTitleText(keyExists )
        }
        if let keyExists = uiStrings["docRetakeButtonText"] as? String{
            getDocConfig().textConfig.setDocRetakePageRetakeButtonText(keyExists)
        }
    }
    
    public func start(context: UIViewController, callback: @escaping ((HVError?,HVResponse?,UIViewController)->Void)) {
        HVDocsViewController.start(context, hvDocConfig: getDocConfig(), completionHandler: callback)
    }
    
    
    public func getDocumentSide(documentSide: String) -> HVDocConfig.DocumentSide {
        switch documentSide {
        case "FRONT", "front":
            return HVDocConfig.DocumentSide.front
        case "BACK", "back":
            return HVDocConfig.DocumentSide.back
        default:
            return HVDocConfig.DocumentSide.front
        }
    }
    
    public func getDocumentType(document: String) ->HyperSnapParams.DocumentType {
        switch document.lowercased() {
        case "a4":
            return HyperSnapParams.DocumentType.a4
        case "card":
            return HyperSnapParams.DocumentType.card
        case "other":
            return HyperSnapParams.DocumentType.other
        case "passport":
            return HyperSnapParams.DocumentType.passport
        default:
            return HyperSnapParams.DocumentType.card
        }
    }
    
    
    
}

