
import Foundation
import HyperSnapSDK

public class FLHVFaceCapture{
    
    var faceConfig = HVFaceConfig()
    
    
    public func getFaceConfig() -> HVFaceConfig {
        return faceConfig
    }
    
    public func createNewFaceConfig() {
        faceConfig = HyperSnapSDK.HVFaceConfig()
    }
    
    public func setHVFaceConfig(hvFaceConfig: Dictionary<String, Any>) {
        if let shouldCheckActiveLiveness = (hvFaceConfig[FLHyperSnapConstants.CHECK_ACTIVE_LIVENESS] as? Bool){
            getFaceConfig().setShouldCheckForActiveLiveness(shouldCheckActiveLiveness)
            if(shouldCheckActiveLiveness) {
                if let totalGestures = (hvFaceConfig[FLHyperSnapConstants.TOTAL_GESTURES] as? Int){
                    getFaceConfig().setTotalGestures(totalGestures)
                }
            }
        }
        if let shouldShowInstructionPage = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_INSTRUCTION_PAGE] as? Bool){
            getFaceConfig().setShouldShowInstructionsPage(shouldShowInstructionPage)
        }
        if let shouldReturnFullImageUrl = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_RETURN_FULL_IMAGE_URL] as? Bool){
            getFaceConfig().setShouldReturnFullImageUri(shouldReturnFullImageUrl)
        }
        if let shouldShowCameraSwitchButton = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_CAMERA_SWITCH_BUTTON] as? Bool){
            getFaceConfig().setShouldShowCameraSwitchButton(shouldShowCameraSwitchButton)
        }
        if let shouldUseBackCamera = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_USE_BACK_CAMERA] as? Bool){
            getFaceConfig().setShouldUseBackCamera(shouldUseBackCamera)
        }
        if let shouldAddPadding = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_ADD_PADDING] as? Bool){
            getFaceConfig().setShouldEnablePadding(shouldAddPadding)
        }
        if let shouldAutoCapture = (hvFaceConfig[FLHyperSnapConstants.FACE_AUTO_CAPTURE] as? Bool){
            if (shouldAutoCapture) {
                if let duration = (hvFaceConfig[FLHyperSnapConstants.AUTO_CAPTURE_DURATION] as? Int){
                    getFaceConfig().setAutoCaptureDuration(duration)
                }
            }
            getFaceConfig().setShouldAutoCapture(shouldAutoCapture)
        }
        
        if let shouldShowTrustLogos = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_TRUST_LOGOS] as? Bool){
            getFaceConfig().setShowTrustLogos(shouldShowTrustLogos)
        }
        if let shouldHandleRetries = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_HANDLE_RETRIES] as? Bool){
            getFaceConfig().setShouldHandleRetries(shouldHandleRetries)
        }
        if let shouldShowCloseAlert = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_CLOSE_ALERT] as? Bool){
            getFaceConfig().setShouldShowCloseAlert(shouldShowCloseAlert)
            if (shouldShowCloseAlert){
                if let closeAlertDialogTitle = (hvFaceConfig[FLHyperSnapConstants.FACE_CLOSE_ALERT_DIALOG_TITLE] as? String){
                    if(!closeAlertDialogTitle.isEmpty){
                        getFaceConfig().textConfig.setCloseAlertTitle(closeAlertDialogTitle)
                    }
                }
                if let closeAlertDialogDescription = (hvFaceConfig[FLHyperSnapConstants.FACE_CLOSE_ALERT_DIALOG_DESC] as? String){
                    if(!closeAlertDialogDescription.isEmpty){
                        getFaceConfig().textConfig.setCloseAlertDesc(closeAlertDialogDescription)
                    }
                }
            }
        }
        if let uiStrings = (hvFaceConfig[FLHyperSnapConstants.FACE_CAPTURE_UI_STRINGS] as? Dictionary<String, Any>){
            if (!uiStrings.isEmpty){
                setUIStrings(uiStrings: uiStrings)
            }
        }
        if let livenessApiDetails = (hvFaceConfig[FLHyperSnapConstants.LIVENESS_API_DETAILS] as? Dictionary<String, Any>){
            if (!livenessApiDetails.isEmpty){
                if let livenessEndpoint = livenessApiDetails[FLHyperSnapConstants.LIVENESS_ENDPOINT] as? String{
                    setLivenessEndpoint(livenessEndpoint:livenessEndpoint )
                }
                if let livenessValue = livenessApiDetails[FLHyperSnapConstants.LIVENESS_VALUE] as? String{
                    setLivenessMode(liveness: livenessValue)
                }
                if let livenessParams = livenessApiDetails[FLHyperSnapConstants.LIVENESS_PARAMS] as? Dictionary<String, Any>{
                    setLivenessAPIParameters(params: livenessParams)
                }
                if let livenessHeaders = livenessApiDetails[FLHyperSnapConstants.LIVENESS_HEADERS] as? Dictionary<String, Any>{
                    setLivenessAPIHeaders(headers: livenessHeaders)
                }
            }
        }
        if let faceVideoRecording = (hvFaceConfig[FLHyperSnapConstants.FACE_VIDEO_RECORDING] as? Dictionary<String, Any>){
            if (!faceVideoRecording.isEmpty){
                if let shouldRecordVideo = faceVideoRecording[FLHyperSnapConstants.SHOULD_RECORD_VIDEO] as? Bool{
                    let noOfFrames = faceVideoRecording[FLHyperSnapConstants.NO_OF_FRAMES] as? Int
                    let fps = faceVideoRecording[FLHyperSnapConstants.FPS] as? Int
                    let bitRateM = faceVideoRecording[FLHyperSnapConstants.BIT_RATE_M] as? Double
                    setVideoRecording(shouldRecordVideo: shouldRecordVideo, numberOfFrames: noOfFrames, bitrateM: bitRateM, fps: fps)
                }
            }
        }
        if let facePadding = (hvFaceConfig[FLHyperSnapConstants.FACE_PADDING] as? Dictionary<String, Any>){
            if (!facePadding.isEmpty){
                let paddingLeft = facePadding[FLHyperSnapConstants.PADDING_LEFT] as! Double
                let paddingRight = facePadding[FLHyperSnapConstants.PADDING_RIGHT] as! Double
                let paddingTop = facePadding[FLHyperSnapConstants.PADDING_TOP] as! Double
                let paddingBottom = facePadding[FLHyperSnapConstants.PADDING_BOTTOM] as! Double
                setPadding(leftPadding: paddingLeft, rightPadding: paddingRight, topPadding: paddingTop, bottomPadding: paddingBottom)
            }
        }
    }
    
    
    public func setLivenessEndpoint(livenessEndpoint: String) {
        getFaceConfig().setLivenessEndpoint(livenessEndpoint)
    }
    
    public func setLivenessMode(liveness: String) {
        var livenessMode: HyperSnapParams.LivenessMode?
        switch liveness {
        case "NONE":
            livenessMode = HyperSnapParams.LivenessMode.none
        case "TEXTURELIVENESS":
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        default:
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        }
        
        getFaceConfig().setLivenessMode(livenessMode!)
    }
    
    public func setLivenessAPIParameters(params: Dictionary<String, Any>) {
        getFaceConfig().setLivenessAPIParameters(params as [String : AnyObject])
    }
    
    public func setLivenessAPIHeaders(headers: Dictionary<String, Any>) {
        getFaceConfig().setLivenessAPIHeaders(headers as! [String : String])
    }
    
    public func setPadding(leftPadding: Double, rightPadding: Double, topPadding: Double, bottomPadding: Double) {
        getFaceConfig().setPadding(left: Float(leftPadding), right: Float(rightPadding), top: Float(topPadding), bottom: Float(bottomPadding))
        
    }
    
    public func setVideoRecording(shouldRecordVideo:Bool,numberOfFrames: Int?,bitrateM: Double?, fps: Int?) {
        if (shouldRecordVideo) {
            if(numberOfFrames != nil){
                getFaceConfig().setNumberOfFrames(numberOfFrames!)
            }
            if(fps != nil){
                getFaceConfig().setFps(fps!)
            }
        }
        
        getFaceConfig().setShouldRecordVideo(shouldRecordVideo)
    }
    
    public func setUIStrings(uiStrings: Dictionary<String,Any>) {
        if let keyExists = uiStrings["faceInstructionsTitle"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsTitle(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsDesc"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsDescription(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsTop1"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsTop1(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsTop2"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsTop2(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsNoGlasses"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsNoGlasses(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsBrightLight"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsBrightLight(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsNoHat"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsNoHat(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsProceed"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsProceed(keyExists)
        }
        if let keyExists = uiStrings["faceInstructionsProceedBackCam"] as? String{
            getFaceConfig().textConfig.setFaceInstructionsProceedBackCam(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureTitle"] as? String{
            getFaceConfig().textConfig.setFaceCaptureTitle(keyExists)
            getFaceConfig().textConfig.setFaceCaptureTitle2(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureSubtitle"] as? String{
            getFaceConfig().textConfig.setFaceCaptureSubtitle(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureMoveCloser"] as? String{
            getFaceConfig().textConfig.setFaceCaptureMoveAwayText2(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureFaceFound"] as? String{
            getFaceConfig().textConfig.setFaceCaptureFaceFoundText(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureFaceNotFound"] as? String{
            getFaceConfig().textConfig.setFaceCaptureFaceNotFoundText(keyExists)
            getFaceConfig().textConfig.setFaceCaptureFaceNotFoundText2(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureMoveAway"] as? String{
            getFaceConfig().textConfig.setFaceCaptureMoveAwayText(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureMultipleFaces"] as? String{
            getFaceConfig().textConfig.setFaceCaptureMultipleFacesDetectedText(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureAutoCaptureWait"] as? String{
            getFaceConfig().textConfig.setFaceAutoCaptureWaitText(keyExists)
        }
        if let keyExists = uiStrings["faceCaptureAutoCaptureAction"] as? String{
            getFaceConfig().textConfig.setFaceAutoCaptureActionText(keyExists)
        }
        if let keyExists = uiStrings["faceLoaderTitle"] as? String{
            getFaceConfig().textConfig.setFaceLoadingTitleText(keyExists)
        }
        if let keyExists = uiStrings["faceLoaderDesc"] as? String{
            getFaceConfig().textConfig.setFaceLoadingDescriptionText(keyExists)
        }
        if let keyExists = uiStrings["faceRetakeTitleText"] as? String{
            getFaceConfig().textConfig.setFaceRetakeTitleText(keyExists)
        }
        if let keyExists = uiStrings["faceRetakeButtonText"] as? String{
            getFaceConfig().textConfig.setFaceRetakeButtonText(keyExists)
        }
        if let keyExists = uiStrings["faceGesture_lookLeft"] as? String{
            getFaceConfig().textConfig.setTextConfig(["faceGesture_lookLeft": keyExists])
        }
        if let keyExists = uiStrings["faceGesture_lookStraight"] as? String{
            getFaceConfig().textConfig.setTextConfig(["faceGesture_lookStraight": keyExists])
        }
        if let keyExists = uiStrings["faceGesture_lookRight"] as? String{
            getFaceConfig().textConfig.setTextConfig(["faceGesture_lookRight": keyExists])
        }
    }
    
    
    
    public func start(context: UIViewController, callback: @escaping ((HVError?,HVResponse?,UIViewController)->Void)) {
        HVFaceViewController.start(context, hvFaceConfig: getFaceConfig(), completionHandler: callback)
    }
}
