import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hypersnapsdk_flutter/hyper_snap_sdk_constants.dart';

import 'hv_result.dart';

/// HVBarcodeScanCapture - used to config and start barcode scan capture
class HVBarcodeScanCapture {
  static const MethodChannel _hvBarcodeScanCaptureChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// start() is used to start barcode scan activity/view inside the client's application
  ///
  /// params :
  ///       hvBarcodeConfig(optional) : type - HVBarcodeConfig
  ///       onComplete : type - Function(HVResponse?,HVError?)
  ///
  /// returns : void
  static start(
      {HVBarcodeConfig? hvBarcodeConfig,
      required Function(HVResponse?, HVError?) onComplete}) async {
    Map resMap = {};
    HVError? hvError;
    HVResponse? hvResponse;

    try {
      resMap = await _hvBarcodeScanCaptureChannel
          .invokeMethod(HyperSnapSDKConstants.qrScanCaptureStart, {
        HyperSnapSDKConstants.hvQRConfig:
            hvBarcodeConfig?.toMap() ?? HVBarcodeConfig().toMap()
      });

      Map errorObj = resMap["errorObj"];
      Map resObj = resMap["resultObj"];
      hvError = (errorObj.isEmpty) ? null : HVError.fromMap(errorObj);
      hvResponse = (resObj.isEmpty) ? null : HVResponse.fromMap(resObj);
    } catch (e) {
      log(e.toString());
    }

    onComplete(hvResponse, hvError);
  }
}

/// HVBarcodeConfig - used to set configurations to Barcode Capture
class HVBarcodeConfig {
  bool? shouldShowInstructionPage;
  HVBarcodeScanUIStrings? hvBarcodeScanUIStrings;
  /// HVBarCodeConfig - used to set configurations to barcode Capture
  ///
  /// params :
  ///        shouldShowInstructionPage - type - bool , used to show instruction page before barcode capture
  ///        hvBarcodeScanUIStrings - type - HVBarcodeScanUIStrings , used to set ui strings for barcode scan screens
  HVBarcodeConfig({this.shouldShowInstructionPage,this.hvBarcodeScanUIStrings});

  Map toMap() {
    return {
      HyperSnapSDKConstants.qrShouldShowInstructionPage:
          shouldShowInstructionPage,
      HyperSnapSDKConstants.qrScanUIStrings : hvBarcodeScanUIStrings?.toMap()
    };
  }
}

class HVBarcodeScanUIStrings {
  String? qrCaptureSubText;
  String? qrCaptureTitleText;
  String? qrCaptureDescText;
  String? qrInstructionsProceedText;
  String? qrInstructionsDescText;
  String? qrInstructionTitleText;
  String? qrCaptureSkipText;

  ///HVBarcodeScanUIStrings - used to set custom strings to all customisable text in HyperSnapSDK face capture
  HVBarcodeScanUIStrings({
    this.qrCaptureDescText,
    this.qrCaptureSkipText,
    this.qrCaptureSubText,
    this.qrCaptureTitleText,
    this.qrInstructionsDescText,
    this.qrInstructionsProceedText,
    this.qrInstructionTitleText
  });

  Map<String, dynamic> toMap(){
    return {
      HyperSnapSDKConstants.qrCaptureDescText : qrCaptureDescText,
      HyperSnapSDKConstants.qrCaptureSkipText : qrCaptureSkipText,
      HyperSnapSDKConstants.qrCaptureSubText : qrCaptureSubText,
      HyperSnapSDKConstants.qrCaptureTitleText : qrCaptureTitleText,
      HyperSnapSDKConstants.qrInstructionsDescText : qrInstructionsDescText,
      HyperSnapSDKConstants.qrInstructionsProceedText : qrInstructionsProceedText,
      HyperSnapSDKConstants.qrInstructionTitleText : qrInstructionTitleText,
    };
  }
}