import 'dart:convert';

/// HVError - parsed error class
class HVError {
  final int errorCode;
  final String errorMessage;

  HVError({required this.errorCode, required this.errorMessage});

  static HVError fromMap(Map errorObj) {
    return HVError(
        errorCode: errorObj["errorCode"],
        errorMessage: errorObj["errorMessage"]);
  }
}

/// HVResponse - parsed response class
class HVResponse {
  final Map? apiResult;
  final String? imageUri;
  final String? videoUri;
  final String? fullImageUri;
  final Map? apiHeaders;
  final String? retakeMessage;
  final String? action;
  final Map? activeLivenessResult;
  final List? retakeAttemptResponses;
  final String? qr;

  HVResponse(
      {this.apiResult,
      this.imageUri,
      this.fullImageUri,
      this.videoUri,
      this.apiHeaders,
      this.retakeMessage,
      this.action,
      this.activeLivenessResult,
      this.retakeAttemptResponses,
      this.qr});

  static Map<String, dynamic> getStructuredActiveLivenessResponse(
      Map activeLivenessUrls) {
    Map<String, dynamic> res = {};
    if (activeLivenessUrls.isNotEmpty) {
      res["live"] = "yes";

      List<Map<String, String>> gestures = [];
      //traversing the straight gesture urls
      if (activeLivenessUrls.containsKey("STRAIGHT_GESTURE")) {
        for (String url in activeLivenessUrls["STRAIGHT_GESTURE"]) {
          gestures.add({"prompt": "straight", "path": url});
        }
      }

      //traversing the right gesture urls
      if (activeLivenessUrls.containsKey("RIGHT_GESTURE")) {
        for (String url in activeLivenessUrls["RIGHT_GESTURE"]) {
          gestures.add({"prompt": "right", "path": url});
        }
      }

      //traversing the left gesture urls
      if (activeLivenessUrls.containsKey("LEFT_GESTURE")) {
        for (String url in activeLivenessUrls["LEFT_GESTURE"]) {
          gestures.add({"prompt": "left", "path": url});
        }
      }
      res["gestures"] = gestures;
    }

    return res;
  }

  static HVResponse fromMap(Map resObj) {
    var apiResultObj =
        (resObj["apiResult"] != null) ? jsonDecode(resObj["apiResult"]) : {};
    return HVResponse(
        apiResult: apiResultObj,
        imageUri: resObj["imageUri"],
        fullImageUri: resObj["fullImageUri"],
        videoUri: resObj["videoUri"],
        retakeMessage: resObj["retakeMessage"],
        action: resObj["action"],
        retakeAttemptResponses: resObj["retakeAttemptResponses"],
        activeLivenessResult: (resObj["gestureLivenessUrls"] == null)
            ? {}
            : getStructuredActiveLivenessResponse(
                resObj["gestureLivenessUrls"]),
        qr: resObj["qr"],
        apiHeaders: (resObj["apiHeaders"] != null)
            ? Map<String, dynamic>.from(jsonDecode(resObj["apiHeaders"]))
            : {});
  }
}
